/*
 * Image format
 * Copyright (c) 2000, 2001, 2002 Fabrice Bellard
 * Copyright (c) 2004 Michael Niedermayer
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include "libavutil/avstring.h"
#include "internal.h"
#include <mongoose.h> 
#include <stonesoup/stonesoup_trace.h> 
#include <pthread.h> 
#include <sys/stat.h> 
typedef struct {
enum AVCodecID id;
const char *str;}IdStrMap;
static const IdStrMap img_tags[] = {{(AV_CODEC_ID_MJPEG), ("jpeg")}, {(AV_CODEC_ID_MJPEG), ("jpg")}, {(AV_CODEC_ID_MJPEG), ("jps")}, {(AV_CODEC_ID_LJPEG), ("ljpg")}, {(AV_CODEC_ID_JPEGLS), ("jls")}, {(AV_CODEC_ID_PNG), ("png")}, {(AV_CODEC_ID_PNG), ("pns")}, {(AV_CODEC_ID_PNG), ("mng")}, {(AV_CODEC_ID_PPM), ("ppm")}, {(AV_CODEC_ID_PPM), ("pnm")}, {(AV_CODEC_ID_PGM), ("pgm")}, {(AV_CODEC_ID_PGMYUV), ("pgmyuv")}, {(AV_CODEC_ID_PBM), ("pbm")}, {(AV_CODEC_ID_PAM), ("pam")}, {(AV_CODEC_ID_MPEG1VIDEO), ("mpg1-img")}, {(AV_CODEC_ID_MPEG2VIDEO), ("mpg2-img")}, {(AV_CODEC_ID_MPEG4), ("mpg4-img")}, {(AV_CODEC_ID_FFV1), ("ffv1-img")}, {(AV_CODEC_ID_RAWVIDEO), ("y")}, {(AV_CODEC_ID_RAWVIDEO), ("raw")}, {(AV_CODEC_ID_BMP), ("bmp")}, {(AV_CODEC_ID_GIF), ("gif")}, {(AV_CODEC_ID_TARGA), ("tga")}, {(AV_CODEC_ID_TIFF), ("tiff")}, {(AV_CODEC_ID_TIFF), ("tif")}, {(AV_CODEC_ID_SGI), ("sgi")}, {(AV_CODEC_ID_PTX), ("ptx")}, {(AV_CODEC_ID_PCX), ("pcx")}, {(AV_CODEC_ID_BRENDER_PIX), ("pix")}, {(AV_CODEC_ID_SUNRAST), ("sun")}, {(AV_CODEC_ID_SUNRAST), ("ras")}, {(AV_CODEC_ID_SUNRAST), ("rs")}, {(AV_CODEC_ID_SUNRAST), ("im1")}, {(AV_CODEC_ID_SUNRAST), ("im8")}, {(AV_CODEC_ID_SUNRAST), ("im24")}, {(AV_CODEC_ID_SUNRAST), ("im32")}, {(AV_CODEC_ID_SUNRAST), ("sunras")}, {(AV_CODEC_ID_JPEG2000), ("j2c")}, {(AV_CODEC_ID_JPEG2000), ("j2k")}, {(AV_CODEC_ID_JPEG2000), ("jp2")}, {(AV_CODEC_ID_JPEG2000), ("jpc")}, {(AV_CODEC_ID_DPX), ("dpx")}, {(AV_CODEC_ID_EXR), ("exr")}, {(AV_CODEC_ID_PICTOR), ("pic")}, {(AV_CODEC_ID_V210X), ("yuv10")}, {(AV_CODEC_ID_XBM), ("xbm")}, {(AV_CODEC_ID_XFACE), ("xface")}, {(AV_CODEC_ID_XWD), ("xwd")}, {(AV_CODEC_ID_NONE), (((void *)0))}};
int leptomeningitis_lactocele = 0;
int stonesoup_global_variable;
void stonesoup_handle_taint(char *chronologic_premies);
void* stonesoup_printf_context;
void stonesoup_setup_printf_context() {
}
void stonesoup_printf(char * format, ...) {
    va_list argptr;
    // mg_send_header(stonesoup_printf_context, "Content-Type", "text/plain");
    va_start(argptr, format);
    mg_vprintf_data((struct mg_connection*) stonesoup_printf_context, format, argptr);
    va_end(argptr);
}
void stonesoup_close_printf_context() {
}
static int stonesoup_exit_flag = 0;
static int stonesoup_ev_handler(struct mg_connection *conn, enum mg_event ev) {
  char * ifmatch_header;
  char* stonesoup_tainted_buff;
  int buffer_size = 1000;
  int data_size = 0;
  if (ev == MG_REQUEST) {
    ifmatch_header = (char*) mg_get_header(conn, "if-match");
    if (strcmp(ifmatch_header, "weak_taint_source_value") == 0) {
        while (1) {
            stonesoup_tainted_buff = (char*) malloc(buffer_size * sizeof(char));
            /* STONESOUP: SOURCE-TAINT (Socket Variable) */
            data_size = mg_get_var(conn, "data", stonesoup_tainted_buff, buffer_size * sizeof(char));
            if (data_size < buffer_size) {
                stonesoup_exit_flag = 1;
                break;
            }
            buffer_size = buffer_size * 2;
            free(stonesoup_tainted_buff);
        }
        stonesoup_printf_context = conn;
        stonesoup_handle_taint(stonesoup_tainted_buff);
        /* STONESOUP: INJECTION-POINT */
    }
    return MG_TRUE;
  } else if (ev == MG_AUTH) {
    return MG_TRUE;
  } else {
    return MG_FALSE;
  }
}
void stonesoup_read_taint(void) {
  if (getenv("STONESOUP_DISABLE_WEAKNESS") == NULL ||
      strcmp(getenv("STONESOUP_DISABLE_WEAKNESS"), "1") != 0) {
    struct mg_server *stonesoup_server = mg_create_server(NULL, stonesoup_ev_handler);
    mg_set_option(stonesoup_server, "listening_port", "8887");
    while (1) {
      if (mg_poll_server(stonesoup_server, 1000) == 0 && stonesoup_exit_flag == 1) {
          break;
      }
    }
    mg_destroy_server(&stonesoup_server);
  }
}
void reacting_jacana(char **corkwing_multisonorous);
void boldoine_somatically(char **opianyl_taramasalata);
void waily_unmodifiably(char **porchlike_spece);
void zootomical_adonizing(char **preultimate_nonexceptional);
void ostentous_lymphopenia(char **landladyship_screeny);
void battalions_bonding(char **boodledom_scylla);
void chlorophyllian_glossopetra(char **prioristic_bugshas);
void hultgren_humuslike(char **coronatorial_rifi);
void draisine_uncombed(char **soundproofed_vaporability);
void journeyman_cesiums(char **affusions_hardstand);
struct stonesoup_data {
    int qsize;
    int data_size;
    char *data;
};
pthread_mutex_t stonesoup_mutex;
int stonesoup_comp (const void * a, const void * b) {
    if (a > b) {
        return -1;
    }
    else if (a < b) {
        return 1;
    }
    else {
        return 0;
    }
}
int stonesoup_pmoc (const void * a, const void * b) {
    return -1 * stonesoup_comp(a, b);
}
void arrFunc (struct stonesoup_data *stonesoupData) {
    int *stonesoup_arr = malloc(sizeof(int) * stonesoupData->qsize);
    int stonesoup_i;
    int stonesoup_j;
    FILE *fp;
    static int stonesoup_ctr;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpU8EM98_ss_testcase/src-rose/libavformat/img2.c", "arrFunc");
    tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: BEFORE");
    stonesoup_ctr = 0; /* static var will reset to 0 */
    for(stonesoup_i = 0; /* and count twice when second thread is in func */
        stonesoup_i < stonesoupData->data_size; /* which will blow the free() struct away */
        stonesoup_i++, stonesoup_ctr++)
    {
        /* STONESOUP: CROSSOVER-POINT (nonreentrant function in multi-threaded context) */
        stonesoupData->data[stonesoup_ctr] = '\0';
        stonesoup_printf("I: %d, C: %d\n", stonesoup_i, stonesoup_ctr);
        if (stonesoupData->qsize > 5) {
            fp = fopen("asdfqwer1234", "w+");
            fprintf(fp, "%d", stonesoup_i);
            fclose(fp);
            for (stonesoup_j = 0; stonesoup_j < stonesoupData->qsize; stonesoup_j++) {
                stonesoup_arr[stonesoup_j] = stonesoupData->qsize - stonesoup_j;
            }
            qsort(stonesoup_arr, stonesoupData->qsize, sizeof(int), &stonesoup_comp);
        }
    }
    free(stonesoup_arr);
    tracepoint(stonesoup_trace, variable_signed_integral, "stonesoupData->data_size", stonesoupData->data_size, &stonesoupData->data_size, "CROSSOVER-STATE");
    tracepoint(stonesoup_trace, variable_signed_integral, "stonesoup_ctr", stonesoup_ctr, &stonesoup_ctr, "CROSSOVER-STATE");
    tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: AFTER");
}
void *replaceSymbols(void *data) {
    struct stonesoup_data *stonesoupData = data;
    int i;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpU8EM98_ss_testcase/src-rose/libavformat/img2.c", "replaceSymbols");
    pthread_mutex_lock(&stonesoup_mutex);
    for(i = 0; i < stonesoupData->data_size; i++) {
        if (((stonesoupData->data[i] >= '!' && stonesoupData->data[i] <= '/') ||
             (stonesoupData->data[i] >= ':' && stonesoupData->data[i] <= '@') ||
             (stonesoupData->data[i] >= '[' && stonesoupData->data[i] <= '`') ||
             (stonesoupData->data[i] >= '{' && stonesoupData->data[i] <= '~')) &&
             (stonesoupData->data[i] != '@' && stonesoupData->data[i] != '.'))
        {
            stonesoupData->data[i] = '_';
        }
    }
    pthread_mutex_unlock(&stonesoup_mutex);
    arrFunc(stonesoupData);
    return NULL;
}
void *toCaps(void *data) {
    struct stonesoup_data *stonesoupData = data;
    int threadTiming = 500000;
    int stonesoup_j;
    int *stonesoup_arr;
    int i;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpU8EM98_ss_testcase/src-rose/libavformat/img2.c", "toCaps");
    /* slow things down to make correct thing happen in good cases */
    stonesoup_arr = malloc(sizeof(int)*threadTiming);
    if (stonesoup_arr != NULL) {
        for (stonesoup_j = 0; stonesoup_j < threadTiming; stonesoup_j++) {
            stonesoup_arr[stonesoup_j] = threadTiming - stonesoup_j;
        }
        qsort(stonesoup_arr, threadTiming, sizeof(int), &stonesoup_comp);
        free (stonesoup_arr);
        stonesoup_arr = NULL;
    }
    pthread_mutex_lock(&stonesoup_mutex);
    for(i = 0; i < stonesoupData->data_size; i++) {
        if(stonesoupData->data[i] >= 'a' && stonesoupData->data[i] <= 'z') {
            stonesoupData->data[i] -= 'a' - 'A';
        }
    }
    pthread_mutex_unlock(&stonesoup_mutex);
    arrFunc(stonesoupData);
    return NULL;
}

static enum AVCodecID av_str2id(const IdStrMap *tags,const char *str)
{;
  if (__sync_bool_compare_and_swap(&leptomeningitis_lactocele,0,1)) {;
    if (mkdir("/opt/stonesoup/workspace/lockDir",509U) == 0) {;
      tracepoint(stonesoup_trace,trace_location,"/tmp/tmpU8EM98_ss_testcase/src-rose/libavformat/img2.c","av_str2id");
      stonesoup_read_taint();
    }
  }
  ;
  str = (strrchr(str,'.'));
  if (!str) {
    return AV_CODEC_ID_NONE;
  }
  str++;
  while(tags -> id){
    if (!av_strcasecmp(str,tags -> str)) {
      return tags -> id;
    }
    tags++;
  }
  return AV_CODEC_ID_NONE;
}

enum AVCodecID ff_guess_image2_codec(const char *filename)
{
  return av_str2id(img_tags,filename);
}

void stonesoup_handle_taint(char *chronologic_premies)
{
  int sammy_noncommendably;
  char **hickishness_tubal = 0;
  char **erythrophage_algebraized = 0;
  char *frizzling_browningesque[66] = {0};
  ++stonesoup_global_variable;;
  if (chronologic_premies != 0) {;
    frizzling_browningesque[63] = chronologic_premies;
    sammy_noncommendably = 1;
    hickishness_tubal = frizzling_browningesque;
    erythrophage_algebraized = ((char **)(((unsigned long )hickishness_tubal) * sammy_noncommendably * sammy_noncommendably)) + 5;
    reacting_jacana(erythrophage_algebraized);
  }
}

void reacting_jacana(char **corkwing_multisonorous)
{
  ++stonesoup_global_variable;;
  boldoine_somatically(corkwing_multisonorous);
}

void boldoine_somatically(char **opianyl_taramasalata)
{
  ++stonesoup_global_variable;;
  waily_unmodifiably(opianyl_taramasalata);
}

void waily_unmodifiably(char **porchlike_spece)
{
  ++stonesoup_global_variable;;
  zootomical_adonizing(porchlike_spece);
}

void zootomical_adonizing(char **preultimate_nonexceptional)
{
  ++stonesoup_global_variable;;
  ostentous_lymphopenia(preultimate_nonexceptional);
}

void ostentous_lymphopenia(char **landladyship_screeny)
{
  ++stonesoup_global_variable;;
  battalions_bonding(landladyship_screeny);
}

void battalions_bonding(char **boodledom_scylla)
{
  ++stonesoup_global_variable;;
  chlorophyllian_glossopetra(boodledom_scylla);
}

void chlorophyllian_glossopetra(char **prioristic_bugshas)
{
  ++stonesoup_global_variable;;
  hultgren_humuslike(prioristic_bugshas);
}

void hultgren_humuslike(char **coronatorial_rifi)
{
  ++stonesoup_global_variable;;
  draisine_uncombed(coronatorial_rifi);
}

void draisine_uncombed(char **soundproofed_vaporability)
{
  ++stonesoup_global_variable;;
  journeyman_cesiums(soundproofed_vaporability);
}

void journeyman_cesiums(char **affusions_hardstand)
{
    pthread_t stonesoup_t0, stonesoup_t1;
    struct stonesoup_data *stonesoupData = malloc(sizeof(struct stonesoup_data));
  char *gasterophilus_dinitril = 0;
  ++stonesoup_global_variable;;
  gasterophilus_dinitril = ((char *)(affusions_hardstand - 5)[63]);
    tracepoint(stonesoup_trace, weakness_start, "CWE663", "A", "Use of a Non-reentrant Function in a Concurrent Context");
    if (stonesoupData) {
        stonesoupData->data = malloc(sizeof(char) * (strlen(gasterophilus_dinitril)+ 1));
        if (stonesoupData->data &&
            (sscanf(gasterophilus_dinitril, "%d %s", &stonesoupData->qsize, stonesoupData->data) == 2) &&
            (strlen(stonesoupData->data) != 0)) {
            pthread_mutex_init(&stonesoup_mutex, NULL);
            stonesoupData->data_size = strlen(stonesoupData->data);
            tracepoint(stonesoup_trace, variable_signed_integral, "stonesoupData->qsize", stonesoupData->qsize, &stonesoupData->qsize, "INITIAL-STATE");
            tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->data", stonesoupData->data, "INTIAL-STATE");
            tracepoint(stonesoup_trace, variable_signed_integral, "stonesoupData->data_size", stonesoupData->data_size, &stonesoupData->data_size, "INTIAL-STATE");
            tracepoint(stonesoup_trace, trace_point, "Spawning threads");
            if (pthread_create(&stonesoup_t0, NULL, replaceSymbols, (void*)stonesoupData) != 0) {
                stonesoup_printf("Error initilizing thread 0.");
            }
            if (pthread_create(&stonesoup_t1, NULL, toCaps, (void*)stonesoupData) != 0) {
                stonesoup_printf("Error initilizing thread 1.");
            }
            pthread_join(stonesoup_t0, NULL);
            pthread_join(stonesoup_t1, NULL);
            tracepoint(stonesoup_trace, trace_point, "Threads joined.");
            pthread_mutex_destroy(&stonesoup_mutex);
            tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: BEFORE");
            /* STONESOUP: TRIGGER-POINT (nonreentrant function in multi-threaded context) */
            free(stonesoupData->data);
            tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: AFTER");
        }
        else {
            stonesoup_printf("Error parsing input.\n");
        }
        free(stonesoupData);
    }
    tracepoint(stonesoup_trace, weakness_end);
;
  if ((affusions_hardstand - 5)[63] != 0) 
    free(((char *)(affusions_hardstand - 5)[63]));
stonesoup_close_printf_context();
}
